__author__ = "Florian Kaiser"
__copyright__ = "Copyright 2022, Project Aktienbot"
__credits__ = ["Florian Kaiser", "Florian Kellermann", "Linus Eickhof", "Kevin Pauer"]
__license__ = "GPL 3.0"
__version__ = "1.0.1"

import datetime
import os

from apiflask import APIBlueprint, abort
from app.auth import auth
from app.db import database as db
from app.helper_functions import make_response
from app.models import SharePrice
from app.schema import SymbolPriceSchema

share_price_blueprint = APIBlueprint('share_price', __name__, url_prefix='/api')
__location__ = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))


@share_price_blueprint.route('/symbols', methods=['GET'])
@share_price_blueprint.output({}, 200)
@share_price_blueprint.auth_required(auth)
@share_price_blueprint.doc(summary="Returns all transaction symbols", description="Returns all transaction symbols for all users")
def get_transaction_symbols():
    # Get all transaction symbols
    symbols = db.session.execute("SELECT isin FROM `transactions` GROUP BY isin;").all()

    return_symbols = []
    for s in symbols:
        return_symbols.append(s[0])

    return make_response(return_symbols, 200, "Successfully loaded symbols")


@share_price_blueprint.route('/symbol', methods=['POST'])
@share_price_blueprint.output({}, 200)
@share_price_blueprint.input(schema=SymbolPriceSchema)
@share_price_blueprint.auth_required(auth)
@share_price_blueprint.doc(summary="Adds new price for isin", description="Adds new price to database")
def add_symbol_price(data):
    # Check if required data is available
    if not check_if_isin_data_exists(data):
        abort(400, message="ISIN missing")

    if not check_if_price_data_exists(data):
        abort(400, message="Price missing")

    if not check_if_time_data_exists(data):
        abort(400, message="Time missing")

    # Add share price
    share_price = SharePrice(
        isin=data['isin'],
        price=data['price'],
        date=datetime.datetime.strptime(data['time'], '%Y-%m-%dT%H:%M:%S.%fZ'),
    )

    db.session.add(share_price)
    db.session.commit()

    return make_response(share_price.as_dict(), 200, "Successfully added price")


def check_if_isin_data_exists(data):
    if 'isin' not in data:
        return False

    if data['isin'] == "" or data['isin'] is None:
        return False

    return True


def check_if_price_data_exists(data):
    if 'price' not in data:
        return False

    if data['price'] == "" or data['price'] is None:
        return False

    return True


def check_if_time_data_exists(data):
    if 'time' not in data:
        return False

    if data['time'] == "" or data['time'] is None:
        return False

    return True
